@@ -36,8 +36,6 @@
 #include "gnome-dialog-util.h"
 #include "gnome-marshal.h"
 
-#include <libgnomeuiP.h>
-
 enum {
   CLICKED,
   CLOSE,
@@ -130,15 +128,12 @@ static void
 gnome_dialog_init (GnomeDialog *dialog)
 {
   GtkWidget * vbox;
-  GtkWidget * bf;
 
   g_return_if_fail(dialog != NULL);
   g_return_if_fail(GNOME_IS_DIALOG(dialog));
 
-  dialog->_priv = g_new0(GnomeDialogPrivate, 1);
-
-  dialog->_priv->just_hide = FALSE;
-  dialog->_priv->click_closes = FALSE;
+  dialog->just_hide = FALSE;
+  dialog->click_closes = FALSE;
   dialog->buttons = NULL;
 
   /* FIXME:!!!!!!!!!!!!!! */
@@ -151,19 +146,14 @@ gnome_dialog_init (GnomeDialog *dialog)
 		      gnome_preferences_get_dialog_position());
 		      */
   
-  dialog->_priv->accelerators = gtk_accel_group_new();
+  dialog->accelerators = gtk_accel_group_new();
   gtk_window_add_accel_group (GTK_WINDOW(dialog), 
-			      dialog->_priv->accelerators);
-
-  bf = gtk_frame_new (NULL);
-  gtk_frame_set_shadow_type (GTK_FRAME (bf), GTK_SHADOW_OUT);
-  gtk_container_add(GTK_CONTAINER(dialog), bf);
-  gtk_widget_show(bf);
+			      dialog->accelerators);
 
   vbox = gtk_vbox_new(FALSE, GNOME_PAD);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 
 			      GNOME_PAD_SMALL);
-  gtk_container_add(GTK_CONTAINER(bf), vbox);
+  gtk_container_add(GTK_CONTAINER(dialog), vbox);
   gtk_widget_show(vbox);
 
   gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, 
@@ -181,21 +171,19 @@ gnome_dialog_init_action_area (GnomeDialog * dialog)
 {
   GtkWidget * separator;
 
-  if (dialog->_priv->action_area)
+  if (dialog->action_area)
     return;
 
-  dialog->_priv->action_area = gtk_hbutton_box_new ();
-  /* FIXME!!!!!!! */
-  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog->_priv->action_area), 0);
-  /* gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog->_priv->action_area),
-			     gnome_preferences_get_button_layout()); */
+  dialog->action_area = gtk_hbutton_box_new ();
+  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog->action_area), 
+			     GTK_BUTTONBOX_END);
 
-  gtk_button_box_set_spacing (GTK_BOX (dialog->_priv->action_area), 
+  gtk_button_box_set_spacing (GTK_BOX (dialog->action_area), 
 			      GNOME_PAD);
 
-  gtk_box_pack_end (GTK_BOX (dialog->vbox), dialog->_priv->action_area, 
+  gtk_box_pack_end (GTK_BOX (dialog->vbox), dialog->action_area, 
 		    FALSE, TRUE, 0);
-  gtk_widget_show (dialog->_priv->action_area);
+  gtk_widget_show (dialog->action_area);
 
   separator = gtk_hseparator_new ();
   gtk_box_pack_end (GTK_BOX (dialog->vbox), separator, 
@@ -376,7 +364,7 @@ void       gnome_dialog_set_parent     (GnomeDialog * dialog,
     gtk_window_set_position(GTK_WINDOW(dialog),GTK_WIN_POS_NONE);
 
     gdk_window_get_origin (GTK_WIDGET(parent)->window, &x, &y);
-    gdk_window_get_size   (GTK_WIDGET(parent)->window, &w, &h);
+    gdk_drawable_get_size   (GTK_WIDGET(parent)->window, &w, &h);
 
     /* The problem here is we don't know how big the dialog is.
        So "centered" isn't really true. We'll go with 
@@ -443,7 +431,7 @@ void       gnome_dialog_append_button (GnomeDialog * dialog,
 
     button = gtk_button_new_from_stock (button_name);
     GTK_WIDGET_SET_FLAGS (GTK_WIDGET (button), GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (dialog->_priv->action_area), button, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (dialog->action_area), button, TRUE, TRUE, 0);
 
     gtk_widget_grab_default (button);
     gtk_widget_show (button);
@@ -531,7 +519,7 @@ void       gnome_dialog_append_button_with_pixmap (GnomeDialog * dialog,
     gnome_dialog_init_action_area (dialog);    
 
     GTK_WIDGET_SET_FLAGS (GTK_WIDGET (button), GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (dialog->_priv->action_area), button, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (dialog->action_area), button, TRUE, TRUE, 0);
 
     gtk_widget_grab_default (button);
     gtk_widget_show (button);
@@ -610,8 +598,8 @@ gnome_dialog_shutdown_run(GnomeDialog* dialog,
 
   if (runinfo->mainloop)
     {
-      g_main_quit(runinfo->mainloop);
-      g_main_destroy(runinfo->mainloop);
+      g_main_loop_quit (runinfo->mainloop);
+      g_main_loop_unref (runinfo->mainloop);
       runinfo->mainloop = NULL;
     }
 }
@@ -686,8 +674,8 @@ gnome_dialog_run_real(GnomeDialog* dialog, gboolean close_after)
   if ( ! GTK_WIDGET_VISIBLE(GTK_WIDGET(dialog)) )
     gtk_widget_show(GTK_WIDGET(dialog));
 
-  ri.mainloop = g_main_new(FALSE);
-  g_main_run(ri.mainloop);
+  ri.mainloop = g_main_loop_new (NULL, FALSE);
+  g_main_loop_run (ri.mainloop);
 
   g_assert(ri.mainloop == NULL);
   
@@ -838,7 +826,7 @@ void       gnome_dialog_set_close    (GnomeDialog * dialog,
   g_return_if_fail(dialog != NULL);
   g_return_if_fail(GNOME_IS_DIALOG(dialog));
 
-  dialog->_priv->click_closes = click_closes;
+  dialog->click_closes = click_closes;
 }
 
 /**
@@ -859,10 +847,9 @@ void       gnome_dialog_close_hides (GnomeDialog * dialog,
   g_return_if_fail(dialog != NULL);
   g_return_if_fail(GNOME_IS_DIALOG(dialog));
 
-  dialog->_priv->just_hide = just_hide;
+  dialog->just_hide = just_hide;
 }
 
-
 /**
  * gnome_dialog_set_sensitive: Set the sensitivity of a button.
  * @dialog: #GnomeDialog to affect.
@@ -985,7 +972,7 @@ void       gnome_dialog_set_accelerator(GnomeDialog * dialog,
     /*FIXME*/
     gtk_widget_add_accelerator(GTK_WIDGET(list->data),
 			       "clicked",
-			       dialog->_priv->accelerators,
+			       dialog->accelerators,
 			       accelerator_key,
 			       accelerator_mods,
 			       GTK_ACCEL_VISIBLE);
@@ -1040,7 +1027,7 @@ gnome_dialog_button_clicked (GtkWidget   *button,
     if (list->data == button) {
 	      gboolean click_closes;
   
-	      click_closes = GNOME_DIALOG(dialog)->_priv->click_closes;
+	      click_closes = GNOME_DIALOG(dialog)->click_closes;
 
 	      gtk_signal_emit (GTK_OBJECT (dialog), dialog_signals[CLICKED], 
 			       which);
@@ -1115,9 +1102,6 @@ static void gnome_dialog_finalize (GObject *object)
 
   dialog = GNOME_DIALOG(object);
 
-  g_free(dialog->_priv);
-  dialog->_priv = NULL;
-
   if (G_OBJECT_CLASS(parent_class)->finalize)
     (* (G_OBJECT_CLASS(parent_class)->finalize))(object);
 }
@@ -1129,7 +1113,7 @@ void gnome_dialog_close_real(GnomeDialog * dialog)
 
   gtk_widget_hide(GTK_WIDGET(dialog));
 
-  if ( ! dialog->_priv->just_hide ) {
+  if ( ! dialog->just_hide ) {
     gtk_widget_destroy (GTK_WIDGET (dialog));
   }
 }
